# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import datetime
import mptt.fields


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Category',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('created', models.DateTimeField(default=datetime.datetime.now, verbose_name='Created')),
                ('updated', models.DateTimeField(default=datetime.datetime.now, verbose_name='Updated')),
                ('is_deleted', models.BooleanField(default=False, db_index=True, verbose_name='Is deleted')),
                ('lft', models.PositiveIntegerField(editable=False, db_index=True)),
                ('rght', models.PositiveIntegerField(editable=False, db_index=True)),
                ('tree_id', models.PositiveIntegerField(editable=False, db_index=True)),
                ('level', models.PositiveIntegerField(editable=False, db_index=True)),
                ('name', models.CharField(max_length=128, verbose_name='Name')),
                ('slug', models.SlugField(unique=True, verbose_name='Slug')),
                ('parent', mptt.fields.TreeForeignKey(on_delete=models.deletion.CASCADE, related_name='children', blank=True, to='categories.Category', null=True)),
            ],
            options={
                'verbose_name': 'Category',
                'verbose_name_plural': 'Categories',
            },
            bases=(models.Model,),
        ),
    ]



from django.conf.urls import url, include
from budget.transactions import views as budget_transactions_views

urlpatterns = [
    url(r'^$', budget_transactions_views.transaction_list,
        name='budget_transaction_list'),
    url(r'^add/$', budget_transactions_views.transaction_add,
        name='budget_transaction_add'),
    url(r'^edit/(?P<transaction_id>\d+)/$',
        budget_transactions_views.transaction_edit,
        name='budget_transaction_edit'),
    url(r'^delete/(?P<transaction_id>\d+)/$',
        budget_transactions_views.transaction_delete,
        name='budget_transaction_delete'),
]



"""
>>> from django.test import Client
>>> c = Client()

>>> r = c.get('/budget/category/')
>>> r.status_code # /budget/category/
200
>>> r.context[-1]['categories']
[]

>>> r = c.get('/budget/category/add/')
>>> r.status_code # /budget/category/add/
200
>>> type(r.context[-1]['form'])
<class 'budget.categories.forms.CategoryForm'>

>>> r = c.post('/budget/category/add/', {'name': 'Mortgage'})
>>> r.status_code # /budget/category/add/
302
>>> r['Location']
'http://testserver/budget/category/'

>>> r = c.get('/budget/category/')
>>> r.status_code # /budget/category/
200
>>> r.context[-1]['categories']
[<Category: Mortgage>]

>>> r = c.get('/budget/category/edit/mortgage/')
>>> r.status_code # /budget/category/edit/mortgage/
200
>>> type(r.context[-1]['form'])
<class 'budget.categories.forms.CategoryForm'>
>>> r.context[-1]['category']
<Category: Mortgage>

>>> r = c.post('/budget/category/edit/mortgage/', {'name': 'First Mortgage'})
>>> r.status_code # /budget/category/edit/mortgage/
302
>>> r['Location']
'http://testserver/budget/category/'

>>> r = c.get('/budget/category/')
>>> r.status_code # /budget/category/
200
>>> r.context[-1]['categories']
[<Category: First Mortgage>]

>>> r = c.get('/budget/category/delete/mortgage/')
>>> r.status_code # /budget/category/delete/mortgage/
200
>>> r.context[-1]['category']
<Category: First Mortgage>

>>> r = c.post('/budget/category/delete/mortgage/', {'confirmed': 'Yes'})
>>> r.status_code # /budget/category/delete/mortgage/
302
>>> r['Location']
'http://testserver/budget/category/'

>>> r = c.get('/budget/category/')
>>> r.status_code # /budget/category/
200
>>> r.context[-1]['categories']
[]
"""



from django.conf import settings
from django.urls import reverse
from django.core.paginator import Paginator, InvalidPage
from django.http import HttpResponseRedirect, Http404
from django.shortcuts import render, get_object_or_404
from django.template import RequestContext
from budget.categories.models import Category
from budget.categories.forms import CategoryForm


def category_list(request, model_class=Category, template_name='budget/categories/list.html'):
    """
    A list of category objects.

    Templates: ``budget/categories/list.html``
    Context:
        categories
            paginated list of category objects
        paginator
            A Django Paginator instance
        page
            current page of category objects
    """
    categories_list = model_class.active.all()
    try:
        paginator = Paginator(categories_list, getattr(settings, 'BUDGET_LIST_PER_PAGE', 50))
        page = paginator.page(request.GET.get('page', 1))
        categories = page.object_list
    except InvalidPage:
        raise Http404('Invalid page requested.')
    return render(request, template_name, {
        'categories': categories,
        'paginator': paginator,
        'page': page,
    })


def category_add(request, form_class=CategoryForm, template_name='budget/categories/add.html'):
    """
    Create a new category object.

    Templates: ``budget/categories/add.html``
    Context:
        form
            a category form
    """
    if request.POST:
        form = form_class(request.POST)

        if form.is_valid():
            category = form.save()
            return HttpResponseRedirect(reverse('budget_category_list'))
    else:
        form = form_class()
    return render(request, template_name, {
        'form': form,
    })


def category_edit(request, slug, model_class=Category, form_class=CategoryForm, template_name='budget/categories/edit.html'):
    """
    Edit a category object.

    Templates: ``budget/categories/edit.html``
    Context:
        category
            the existing category object
        form
            a category form
    """
    category = get_object_or_404(model_class.active.all(), slug=slug)
    if request.POST:
        form = form_class(request.POST, instance=category)

        if form.is_valid():
            category = form.save()
            return HttpResponseRedirect(reverse('budget_category_list'))
    else:
        form = form_class(instance=category)
    return render(request, template_name, {
        'category': category,
        'form': form,
    })


def category_delete(request, slug, model_class=Category, template_name='budget/categories/delete.html'):
    """
    Delete a category object.

    Templates: ``budget/categories/delete.html``
    Context:
        category
            the existing category object
    """
    category = get_object_or_404(model_class.active.all(), slug=slug)
    if request.POST:
        if request.POST.get('confirmed') and request.POST['confirmed'] == 'Yes':
            category.delete()
        return HttpResponseRedirect(reverse('budget_category_list'))
    return render(request, template_name, {
        'category': category,
    })



from django.conf import settings
from django.urls import reverse
from django.core.paginator import Paginator, InvalidPage
from django.http import Http404, HttpResponseRedirect
from django.shortcuts import render, get_object_or_404
from django.template import RequestContext
from budget.transactions.models import Transaction
from budget.transactions.forms import TransactionForm


def transaction_list(request, model_class=Transaction,
                     template_name='budget/transactions/list.html'):
    """
    A list of transaction objects.

    Templates: ``budget/transactions/list.html``
    Context:
        transactions
            paginated list of transaction objects
        paginator
            A Django Paginator instance
        page
            current page of transaction objects
    """
    transaction_list = model_class.active.order_by('-date', '-created')
    try:
        paginator = Paginator(transaction_list,
                              getattr(settings, 'BUDGET_LIST_PER_PAGE', 50))
        page = paginator.page(request.GET.get('page', 1))
        transactions = page.object_list
    except InvalidPage:
        raise Http404('Invalid page requested.')
    return render(request, template_name, {
        'transactions': transactions,
        'paginator': paginator,
        'page': page,
    })


def transaction_add(request, form_class=TransactionForm,
                    template_name='budget/transactions/add.html'):
    """
    Create a new transaction object.

    Templates: ``budget/transactions/add.html``
    Context:
        form
            a transaction form
    """
    if request.POST:
        form = form_class(request.POST)

        if form.is_valid():
            form.save()
            return HttpResponseRedirect(reverse('budget_transaction_list'))
    else:
        form = form_class()
    return render(request, template_name, {
        'form': form,
    })


def transaction_edit(request, transaction_id, model_class=Transaction,
                     form_class=TransactionForm,
                     template_name='budget/transactions/edit.html'):
    """
    Edit a transaction object.

    Templates: ``budget/transactions/edit.html``
    Context:
        transaction
            the existing transaction object
        form
            a transaction form
    """
    transaction = get_object_or_404(model_class.active.all(),
                                    pk=transaction_id)
    if request.POST:
        form = form_class(request.POST, instance=transaction)

        if form.is_valid():
            form.save()
            return HttpResponseRedirect(reverse('budget_transaction_list'))
    else:
        form = form_class(instance=transaction)
    return render(request, template_name, {
        'transaction': transaction,
        'form': form,
    })


def transaction_delete(request, transaction_id, model_class=Transaction,
                       template_name='budget/transactions/delete.html'):
    """
    Delete a transaction object.

    Templates: ``budget/transactions/delete.html``
    Context:
        transaction
            the existing transaction object
    """
    transaction = get_object_or_404(Transaction.active.all(),
                                    pk=transaction_id)
    if request.POST:
        if request.POST.get('confirmed'):
            transaction.delete()
        return HttpResponseRedirect(reverse('budget_transaction_list'))
    return render(request, template_name, {
        'transaction': transaction,
    })



from django import forms
from django.template.defaultfilters import slugify

#
from mptt.forms import TreeNodeChoiceField

#
from budget.categories.models import Category


class CategoryForm(forms.ModelForm):
    class Meta:
        model = Category
        category_tree = TreeNodeChoiceField(queryset=Category.objects.all())
        fields = ('name',)
    
    def save(self):
        if not self.instance.slug:
            self.instance.slug = slugify(self.cleaned_data['name'])
        super(CategoryForm, self).save()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import datetime


class Migration(migrations.Migration):

    dependencies = [
        ('categories', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Budget',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('created', models.DateTimeField(default=datetime.datetime.now, verbose_name='Created')),
                ('updated', models.DateTimeField(default=datetime.datetime.now, verbose_name='Updated')),
                ('is_deleted', models.BooleanField(default=False, db_index=True, verbose_name='Is deleted')),
                ('name', models.CharField(max_length=255, verbose_name='Name')),
                ('slug', models.SlugField(unique=True, verbose_name='Slug')),
                ('start_date', models.DateTimeField(default=datetime.datetime.now, verbose_name='Start Date', db_index=True)),
            ],
            options={
                'verbose_name': 'Budget',
                'verbose_name_plural': 'Budgets',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='BudgetEstimate',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('created', models.DateTimeField(default=datetime.datetime.now, verbose_name='Created')),
                ('updated', models.DateTimeField(default=datetime.datetime.now, verbose_name='Updated')),
                ('is_deleted', models.BooleanField(default=False, db_index=True, verbose_name='Is deleted')),
                ('amount', models.DecimalField(verbose_name='Amount', max_digits=11, decimal_places=2)),
                ('repeat', models.CharField(blank=True, max_length=15, verbose_name='Repeat', choices=[(b'BIWEEKLY', 'Every 2 Weeks'), (b'MONTHLY', 'Every Month')])),
                ('occurring_month', models.IntegerField(blank=True, max_length=3, null=True, verbose_name='Occuring month', choices=[(1, 'January'), (2, 'February'), (3, 'March'), (4, 'April'), (5, 'May'), (6, 'June'), (7, 'July'), (8, 'August'), (9, 'September'), (10, 'October'), (11, 'November'), (12, 'December')])),
                ('budget', models.ForeignKey(on_delete=models.deletion.CASCADE, related_name='estimates', verbose_name='Budget', to='budget.Budget')),
                ('category', models.ForeignKey(on_delete=models.deletion.CASCADE, related_name='estimates', verbose_name='Category', to='categories.Category')),
            ],
            options={
                'verbose_name': 'Budget estimate',
                'verbose_name_plural': 'Budget estimates',
            },
            bases=(models.Model,),
        ),
    ]



from django.conf.urls import url, include
from budget.categories import views as budget_categories_views
urlpatterns = [
    url(r'^$', budget_categories_views.category_list,
        name='budget_category_list'),
    url(r'^add/$', budget_categories_views.category_add,
        name='budget_category_add'),
    url(r'^edit/(?P<slug>[\w_-]+)/$', budget_categories_views.category_edit,
        name='budget_category_edit'),
    url(r'^delete/(?P<slug>[\w_-]+)/$', budget_categories_views.category_delete,
        name='budget_category_delete'),
]



from django import forms

#
from mptt.forms import TreeNodeChoiceField

#
from budget.transactions.models import Transaction
from budget.categories.models import Category

class TransactionForm(forms.ModelForm):

    # Replace category in the model with hieararchial category list
    category = TreeNodeChoiceField(queryset=Category.objects.all())

    class Meta:
        model = Transaction
        fields = ('transaction_type', 'notes', 'category', 'amount', 'date')



import datetime
from decimal import Decimal
from django.utils.encoding import python_2_unicode_compatible

from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.db.models import Q

# django mptt
from mptt.models import MPTTModel, TreeForeignKey, TreeManager

#

class StandardMetadataMPTT(MPTTModel):
    """
    A basic (abstract) model for metadata.
    """
    created = models.DateTimeField(_('Created'), default=datetime.datetime.now)
    updated = models.DateTimeField(_('Updated'), default=datetime.datetime.now)
    is_deleted = models.BooleanField(_('Is deleted'), default=False, db_index=True)

    class Meta:
        abstract = True

    def save(self, *args, **kwargs):
        self.updated = datetime.datetime.now()
        super(StandardMetadataMPTT, self).save(*args, **kwargs)

    def delete(self):
        self.is_deleted = True
        self.save()


class ActiveManagerMPTT(TreeManager):
    def get_queryset(self):
        return super(ActiveManagerMPTT, self).get_queryset().filter(is_deleted=False)

class CategoryRootNodeManager(TreeManager):
    def get_queryset(self):
        return super(CategoryRootNodeManager, self).get_queryset().filter(level=0)

@python_2_unicode_compatible
class Category(StandardMetadataMPTT):
    """
    Categories are the means to loosely tie together the transactions and
    estimates.

    They are used to aggregate transactions together and compare them to the
    appropriate budget estimate. For the reasoning behind this, the docstring
    on the Transaction object explains this.
    """

    parent = TreeForeignKey('self', on_delete=models.CASCADE, null=True, blank=True, related_name='children', db_index=True)
    name = models.CharField(_('Name'), max_length=128)
    slug = models.SlugField(_('Slug'), unique=True)

    objects = TreeManager()
    active = ActiveManagerMPTT()
    root_nodes = CategoryRootNodeManager()

    class Meta:
        verbose_name = _('Category')
        verbose_name_plural = _('Categories')

    def __str__(self):
        return self.name


    def estimates_and_transactions(self, start_date, end_date, query = Q()):

        transactions = self.transactions.all()
        estimates = self.estimates.all()

def get_queryset_descendants(nodes, include_self=False):
    """
    http://stackoverflow.com/questions/5722767/django-mptt-get-descendants-for-a-list-of-nodes
    """
    if not nodes:
        return Category.objects.none()
    filters = []
    for n in nodes:
        lft, rght = n.lft, n.rght
        if include_self:
            lft -=1
            rght += 1
        filters.append(Q(tree_id=n.tree_id, lft__gt=lft, rght__lt=rght))
    # q = reduce(operator.or_, filters)
    # Removing reduce, as reduced from std library
    # (http://www.artima.com/weblogs/viewpost.jsp?thread=98196 )
    # Do it with for loop instead, just of everything together

    query = Q()
    for filter in filters:
        query |= filter
    return Category.objects.filter(query)



import datetime
import calendar

from django.conf import settings
from django.core.exceptions import ObjectDoesNotExist
from django.core.paginator import Paginator, InvalidPage
from django.urls import reverse

from django.http import Http404, HttpResponseRedirect
from django.shortcuts import render, get_object_or_404
from django.template import RequestContext

from budget.models import Budget, BudgetEstimate
from budget.categories.models import Category, get_queryset_descendants
from budget.transactions.models import Transaction
from budget.forms import BudgetEstimateForm, BudgetForm


def dashboard(request, budget_model_class=Budget,
              transaction_model_class=Transaction,
              template_name='budget/dashboard.html'):
    """
    Provides a high-level rundown of recent activity and budget status.

    Template: ``budget/dashboard.html``
    Context:
        budget
            the most current budget object
        latest_expenses
            the most recent expenses (by default, the last 10)
        latest_incomes
            the most recent incomes (by default, the last 10)
        estimated_amount
            the budget's estimated total for the month
        amount_used
            the actual amount spent for the month
        progress_bar_percent
            the percentage of the budget actually spent so far
    """
    today = datetime.date.today()
    start_date = datetime.date(today.year, today.month, 1)
    end_year, end_month = today.year, today.month + 1

    if end_month > 12:
        end_year += 1
        end_month = 1

    end_date = datetime.date(end_year, end_month, 1) - datetime.timedelta(days=1)

    try:
        budget = budget_model_class.active.most_current_for_date(datetime.datetime.today())
    except ObjectDoesNotExist:
        # Since there are no budgets at this point, pass them on to setup
        # as this view is meaningless without at least basic data in place.
        return HttpResponseRedirect(reverse('budget_setup'))

    latest_expenses = transaction_model_class.expenses.get_latest()
    latest_incomes = transaction_model_class.incomes.get_latest()

    estimated_amount = budget.monthly_estimated_total_current_month(today.month)
    amount_used = budget.actual_total(start_date, end_date)

    if estimated_amount == 0:
        progress_bar_percent = 100
    else:
        progress_bar_percent = int(amount_used / estimated_amount * 100)

    if progress_bar_percent >= 100:
        progress_bar_percent = 100

    return render(request, template_name, {
        'budget': budget,
        'latest_expenses': latest_expenses,
        'latest_incomes': latest_incomes,
        'estimated_amount': estimated_amount,
        'amount_used': amount_used,
        'progress_bar_percent': progress_bar_percent,
    })


def setup(request, template_name='budget/setup.html'):
    """
    Displays a setup page which ties together the
    category/budget/budget estimate areas with explanatory text on how to use
    to set everything up properly.

    Templates: ``budget/setup.html``
    """
    return render(request, template_name, {})


def summary_list(request, transaction_model_class=Transaction,
                 template_name='budget/summaries/summary_list.html'):
    """
    Displays a list of all months that may have transactions for that month.

    Templates: ``budget/summaries/summary_list.html``
    Context:
        dates
            a list of datetime objects representing all years/months that
            have transactions
    """
    dates = transaction_model_class.active.all().dates('date', 'month')
    return render(request, template_name, {
        'dates': dates,
    })


def summary_year_no_months(request, year, budget_model_class=Budget,
                           template_name='budget/summaries/summary_year.html'):
    """
    Displays a budget report for the year to date.

    Templates: ``budget/summaries/summary_year.html``
    Context:
        budget
            the most current budget object for the year
        estimates_and_transactions
            a list of dictionaries containing each budget estimate, the
            corresponding transactions and total amount of the transactions
        actual_total
            the total amount of all transactions represented in the budget
            for the year
        start_date
            the first date for the year
        end_date
            the last date for the year
    """
    start_date = datetime.date(int(year), 1, 1)
    end_date = datetime.date(int(year), 12, 31)
    budget = budget_model_class.active.most_current_for_date(end_date)

    root_nodes = Category.root_nodes.all()
    categories = get_queryset_descendants(root_nodes, include_self=True)

    categories_estimates_and_transactions, actual_total = budget.categories_estimates_and_transactions(start_date, end_date, categories, 'all')
    return render(request, template_name, {
        'budget': budget,
        'categories_estimates_and_transactions': categories_estimates_and_transactions,
        'actual_total': actual_total,
        'year': year,
    })


def summary_year_detail(request, year, budget_model_class=Budget,
                        template_name='budget/summaries/summary_year_months.html'):
    """
    Displays a budget report for the year to date, month by month

    Templates: ``budget/summaries/summary_year_months.html``
    Context:
        budget
            the most current budget object for the year
        estimates_and_transactions
            a list of dictionaries containing each budget estimate, the
            corresponding transactions and total amount of the transactions
        actual_total
            the total amount of all transactions represented in the budget
            for the year
        start_date
            the first date for the year
        end_date
            the last date for the year
    """
    year_end_date = datetime.date(int(year), 12, 31)

    root_nodes = Category.root_nodes.all()
    categories = get_queryset_descendants(root_nodes, include_self=True)
    budget = budget_model_class.active.most_current_for_date(year_end_date)
    yearly_data = budget.yearly_data_per_category(categories, budget,
                                                  year)
    return render(request, template_name, {
        'months': calendar.month_abbr[1:],
        'categories': categories,
        'budget': budget,
        'estimates_and_actuals': yearly_data.estimates_and_actuals,
        'actual_yearly_total': yearly_data.actual_yearly_total,
        'year': year,
    })


def summary_month(request, year, month, budget_model_class=Budget,
                  template_name='budget/summaries/summary_month.html'):
    """
    Displays a budget report for the month to date.

    Templates: ``budget/summaries/summary_month.html``
    Context:
        budget
            the most current budget object for the month
        estimates_and_transactions
            a list of dictionaries containing each budget estimate, the
            corresponding transactions and total amount of the transactions
        actual_total
            the total amount of all transactions represented in the budget
            for the month
        start_date
            the first date for the month
        end_date
            the last date for the month
    """
    start_date = datetime.date(int(year), int(month), 1)
    end_year, end_month = int(year), int(month) + 1

    if end_month > 12:
        end_year += 1
        end_month = 1

    end_date = datetime.date(end_year, end_month, 1) - datetime.timedelta(days=1)
    # Search for monthly expenses and those estimated to happen in this month

    budget = budget_model_class.active.most_current_for_date(end_date)
    estimated_total = budget.monthly_estimated_total_current_month(int(month))

    root_nodes = Category.root_nodes.all()
    categories = get_queryset_descendants(root_nodes, include_self=True)

    categories_estimates_and_transactions, actual_total = budget.categories_estimates_and_transactions(start_date, end_date, categories, month)

    return render(request, template_name, {
        'budget': budget,
        'categories_estimates_and_transactions': categories_estimates_and_transactions,
        'actual_total': actual_total,
        'start_date': start_date,
        'end_date': end_date,
        'estimated_total': estimated_total,
        'categories': categories,
    })


def budget_list(request, model_class=Budget,
                template_name='budget/budgets/list.html'):
    """
    A list of budget objects.

    Templates: ``budget/budgets/list.html``
    Context:
        budgets
            paginated list of budget objects
        paginator
            A Django Paginator instance
        page
            current page of budget objects
    """
    budgets_list = model_class.active.all()
    try:
        paginator = Paginator(budgets_list,
                              getattr(settings, 'BUDGET_LIST_PER_PAGE', 50))
        page = paginator.page(request.GET.get('page', 1))
        budgets = page.object_list
    except InvalidPage:
        raise Http404('Invalid page requested.')
    return render(request, template_name, {
        'budgets': budgets,
        'paginator': paginator,
        'page': page,
    })


def budget_add(request, form_class=BudgetForm,
               template_name='budget/budgets/add.html'):
    """
    Create a new budget object.

    Templates: ``budget/budgets/add.html``
    Context:
        form
            a budget form
    """
    if request.POST:
        form = form_class(request.POST)

        if form.is_valid():
            form.save()
            return HttpResponseRedirect(reverse('budget_budget_list'))
    else:
        form = form_class()
    return render_to_response(request, template_name, {
        'form': form,
    })


def budget_edit(request, slug, model_class=Budget, form_class=BudgetForm,
                template_name='budget/budgets/edit.html'):
    """
    Edit a budget object.

    Templates: ``budget/budgets/edit.html``
    Context:
        budget
            the existing budget object
        form
            a budget form
    """
    budget = get_object_or_404(model_class.active.all(), slug=slug)
    if request.POST:
        form = form_class(request.POST, instance=budget)

        if form.is_valid():
            form.save()
            return HttpResponseRedirect(reverse('budget_budget_list'))
    else:
        form = form_class(instance=budget)
    return render(request, template_name, {
        'budget': budget,
        'form': form,
    })


def budget_delete(request, slug, model_class=Budget,
                  template_name='budget/budgets/delete.html'):
    """
    Delete a budget object.

    Templates: ``budget/budgets/delete.html``
    Context:
        budget
            the existing budget object
    """
    budget = get_object_or_404(model_class.active.all(), slug=slug)
    if request.POST:
        if request.POST.get('confirmed'):
            budget.delete()
        return HttpResponseRedirect(reverse('budget_budget_list'))
    return render(request, template_name, {
        'budget': budget,
    })


def estimate_list(request, budget_slug, budget_model_class=Budget,
                  model_class=BudgetEstimate,
                  template_name='budget/estimates/list.html'):
    """
    A list of estimate objects.

    Templates: ``budget/estimates/list.html``
    Context:
        budget
            the parent budget object for the estimates
        estimates
            paginated list of estimate objects
        paginator
            A Django Paginator instance
        page
            current page of estimate objects
    """
    budget = get_object_or_404(budget_model_class.active.all(),
                               slug=budget_slug)
    estimates_list = model_class.active.filter(budget__id=budget.id).all()
    try:
        paginator = Paginator(estimates_list,
                              getattr(settings, 'BUDGET_LIST_PER_PAGE', 50))
        page = paginator.page(request.GET.get('page', 1))
        estimates = page.object_list
    except InvalidPage:
        raise Http404('Invalid page requested.')
    return render(request, template_name, {
        'budget': budget,
        'estimates': estimates,
        'paginator': paginator,
        'page': page,
    })


def estimate_add(request, budget_slug, budget_model_class=Budget,
                 form_class=BudgetEstimateForm,
                 template_name='budget/estimates/add.html'):
    """
    Create a new estimate object.

    Templates: ``budget/estimates/add.html``
    Context:
        budget
            the parent budget object for the estimate
        form
            a estimate form
    """
    budget = get_object_or_404(budget_model_class.active.all(),
                               slug=budget_slug)
    if request.POST:
        form = form_class(request.POST)

        if form.is_valid():
            form.save(budget=budget)
            return HttpResponseRedirect(
                reverse('budget_estimate_list',
                        kwargs={'budget_slug': budget.slug})
            )
    else:
        form = form_class()
    return render(request, template_name, {
        'budget': budget,
        'form': form,
    })


def estimate_edit(request, budget_slug, estimate_id,
                  budget_model_class=Budget,
                  form_class=BudgetEstimateForm,
                  template_name='budget/estimates/edit.html'):
    """
    Edit a estimate object.

    Templates: ``budget/estimates/edit.html``
    Context:
        budget
            the parent budget object for the estimate
        estimate
            the existing estimate object
        form
            a estimate form
    """
    budget = get_object_or_404(budget_model_class.active.all(),
                               slug=budget_slug)
    try:
        estimate = budget.estimates.get(pk=estimate_id, is_deleted=False)
    except ObjectDoesNotExist:
        raise Http404("The requested estimate could not be found.")
    if request.POST:
        form = form_class(request.POST, instance=estimate)

        if form.is_valid():
            form.save(budget=budget)
            return HttpResponseRedirect(
                reverse('budget_estimate_list',
                        kwargs={'budget_slug': budget.slug})
            )
    else:
        form = form_class(instance=estimate)
    return render(request, template_name, {
        'budget': budget,
        'estimate': estimate,
        'form': form,
    })


def estimate_delete(request, budget_slug, estimate_id,
                    budget_model_class=Budget,
                    template_name='budget/estimates/delete.html'):
    """
    Delete a estimate object.

    Templates: ``budget/estimates/delete.html``
    Context:
        budget
            the parent budget object for the estimate
        estimate
            the existing estimate object
    """
    budget = get_object_or_404(budget_model_class.active.all(),
                               slug=budget_slug)
    try:
        estimate = budget.estimates.get(pk=estimate_id, is_deleted=False)
    except ObjectDoesNotExist:
        raise Http404("The requested estimate could not be found.")
    if request.POST:
        if request.POST.get('confirmed'):
            estimate.delete()
        return HttpResponseRedirect(
            reverse('budget_estimate_list',
                    kwargs={'budget_slug': budget.slug})
        )
    return render(request, template_name, {
        'budget': budget,
        'estimate': estimate,
    })



from django.conf.urls import url, include
from budget import views as budget_views
"""
Needed Urls:
Dashboard
Year Summary
Month Summary
CRUD Budget
CRUD BudgetEstimates

Eventually:
Custom date range
Week Summary
Day Summary
...?
"""

urlpatterns = [
    url(r'^$', budget_views.dashboard, name='budget_dashboard'),
    url(r'^setup/$', budget_views.setup, name='budget_setup'),
    
    # Summaries
    url(r'^summary/$', budget_views.summary_list, name='budget_summary_list'),
    url(r'^summary/(?P<year>\d{4})/simple/$',
        budget_views.summary_year_no_months,
        name='budget_summary_year_no_months'),
    url(r'^summary/(?P<year>\d{4})/$',
        budget_views.summary_year_detail, name='budget_summary_year'),
    url(r'^summary/(?P<year>\d{4})/(?P<month>\d{1,2})/$',
        budget_views.summary_month, name='budget_summary_month'),
    
    # Categories
    url(r'^category/', include('budget.categories.urls')),
    
    # Budget
    url(r'^budget/$', budget_views.budget_list, name='budget_budget_list'),
    url(r'^budget/add/$', budget_views.budget_add, name='budget_budget_add'),
    url(r'^budget/edit/(?P<slug>[\w-]+)/$',
        budget_views.budget_edit, name='budget_budget_edit'),
    url(r'^budget/delete/(?P<slug>[\w-]+)/$',
        budget_views.budget_delete, name='budget_budget_delete'),
    
    # BudgetEstimates
    url(r'^budget/(?P<budget_slug>[\w-]+)/estimate/$',
        budget_views.estimate_list, name='budget_estimate_list'),
    url(r'^budget/(?P<budget_slug>[\w-]+)/estimate/add/$',
        budget_views.estimate_add, name='budget_estimate_add'),
    url(r'^budget/(?P<budget_slug>[\w-]+)/estimate/edit/(?P<estimate_id>\d+)/$',
        budget_views.estimate_edit, name='budget_estimate_edit'),
    url(r'^budget/(?P<budget_slug>[\w-]+)/estimate/delete/(?P<estimate_id>\d+)/$', budget_views.estimate_delete, name='budget_estimate_delete'),
    
    # Transaction
    url(r'^transaction/', include('budget.transactions.urls')),
]



"""
>>> from django.test import Client
>>> c = Client()

>>> from django.core.management import call_command
>>> call_command('loaddata', 'categories_testdata.yaml') #doctest: +ELLIPSIS
Installing yaml fixture 'categories_testdata' from ...
Installed 1 object(s) from 1 fixture(s)

>>> from budget.categories.models import Category
>>> cat = Category.objects.get(slug='misc')

>>> r = c.get('/budget/transaction/')
>>> r.status_code # /budget/transaction/
200
>>> r.context[-1]['transactions']
[]

>>> r = c.get('/budget/transaction/add/')
>>> r.status_code # /budget/transaction/add/
200
>>> type(r.context[-1]['form'])
<class 'budget.transactions.forms.TransactionForm'>

>>> r = c.post('/budget/transaction/add/', {'transaction_type': 'income', 'category': cat.id, 'notes': 'Paycheck', 'amount': '300.00', 'date': '2008-10-14'})
>>> r.status_code # /budget/transaction/add/
302
>>> r['Location']
'http://testserver/budget/transaction/'

>>> r = c.get('/budget/transaction/')
>>> r.status_code # /budget/transaction/
200
>>> r.context[-1]['transactions']
[<Transaction: Paycheck (Income) - 300>]

>>> r = c.get('/budget/transaction/edit/1/')
>>> r.status_code # /budget/transaction/edit/1/
200
>>> type(r.context[-1]['form'])
<class 'budget.transactions.forms.TransactionForm'>
>>> r.context[-1]['transaction']
<Transaction: Paycheck (Income) - 300>

>>> r = c.post('/budget/transaction/edit/1/', {'transaction_type': 'income', 'category': cat.id, 'notes': 'My Paycheck', 'amount': '300.00', 'date': '2008-10-14'})
>>> r.status_code # /budget/transaction/edit/1/
302
>>> r['Location']
'http://testserver/budget/transaction/'

>>> r = c.get('/budget/transaction/')
>>> r.status_code # /budget/transaction/
200
>>> r.context[-1]['transactions']
[<Transaction: My Paycheck (Income) - 300>]

>>> r = c.get('/budget/transaction/delete/1/')
>>> r.status_code # /budget/transaction/delete/1/
200
>>> r.context[-1]['transaction']
<Transaction: My Paycheck (Income) - 300>

>>> r = c.post('/budget/transaction/delete/1/', {'confirmed': 'Yes'})
>>> r.status_code # /budget/transaction/delete/1/
302
>>> r['Location']
'http://testserver/budget/transaction/'

>>> r = c.get('/budget/transaction/')
>>> r.status_code # /budget/transaction/
200
>>> r.context[-1]['transactions']
[]
"""



from django.contrib import admin

from budget.models import Budget, BudgetEstimate

class BudgetAdmin(admin.ModelAdmin):
    date_hierarchy = 'start_date'
    fieldsets = (
        (None, {
            'fields': ('name', 'slug', 'start_date'),
        }),
        ('Metadata', {
            'classes': ('collapse',),
            'fields': ('created', 'updated', 'is_deleted')
        })
    )
    list_display = ('name', 'start_date', 'is_deleted')
    list_filter = ('is_deleted',)
    prepopulated_fields = {
        'slug': ('name',),
    }
    search_fields = ('name',)


class BudgetEstimateAdmin(admin.ModelAdmin):
    fieldsets = (
        (None, {
            'fields': ('budget', 'category', 'amount'),
        }),
        ('Metadata', {
            'classes': ('collapse',),
            'fields': ('created', 'updated', 'is_deleted')
        })
    )
    list_display = ('category', 'budget', 'amount', 'is_deleted')
    list_filter = ('is_deleted',)


admin.site.register(Budget, BudgetAdmin)
admin.site.register(BudgetEstimate, BudgetEstimateAdmin)



import datetime
from decimal import Decimal
import calendar
from collections import namedtuple

from django.db import models
from django.db.models import Q

from multiselectfield import MultiSelectField

from budget.base_models import ActiveManager, StandardMetadata
from budget.categories.models import Category
from budget.transactions.models import Transaction
from django.utils.translation import ugettext_lazy as _


class BudgetManager(ActiveManager):
    def most_current_for_date(self, date):
        return super(BudgetManager, self).get_queryset().filter(start_date__lte=date).latest('start_date')


class Budget(StandardMetadata):
    """
    An object representing a budget.

    Only estimates are tied to a budget object, which allows different budgets
    to be applied to the same set of transactions for comparision.
    """
    name = models.CharField(_('Name'), max_length=255)
    slug = models.SlugField(_('Slug'), unique=True)
    start_date = models.DateTimeField(_('Start Date'),
                                      default=datetime.datetime.now,
                                      db_index=True)

    objects = models.Manager()
    active = BudgetManager()

    def __unicode__(self):
        return self.name

    def yearly_estimated_total(self):
        """
        Sum of estimates for the current budget
        Budget is assumed to cover entire year
        """
        total = Decimal('0.0')
        for estimate in self.estimates.exclude(is_deleted=True):
            for month in estimate.occurring_month:
                total += estimate.amount
        return total

    def monthly_estimated_total_current_month(self, month):
        total = Decimal('0.0')
        for estimate in self.estimates.exclude(is_deleted=True):
            if str(month) in estimate.occurring_month:
                total += estimate.amount
        return total

    def categories_estimates_and_transactions(self, start_date, end_date,
                                              categories,
                                              month):

        """Return data for all the categories in given month, or entire
        year

        Args:
           start_date (datetime.datetime): Start of the date range
           end_date (datetime.datetime): End of the date range
           categories (mptt.TreeQuerySet): List of categories
           month (str): number of month as string, or 'all' for entire year

        Returns:
           A list. First item is list of dictionaries, each dictiory contains
                   - category (Category)
                   - estimate (BudgetEstimate)
                   - transactions (Queryset): Transactions for given period
                   - actual_amount (Decimal): Actual spent amount for the
                     estimate
                   The second item is total sum spent for the category in
                   the given time range
        """

        categories_estimates_and_transactions = []

        actual_total = Decimal('0.0')
        for category in categories:
            actual_amount = Decimal('0.0')
            estimate_found = False

            # Search for each category, and where occurence match
            query_list = Q(category=category)
            transactions =  Transaction.expenses.filter(query_list).order_by('date')
            for estimate in self.category_estimates(query_list):
                # As occurring_month is a string of comma separated list of
                # (month) number, there doesn't seem to be a way to make a
                # query that would return estimates having e.g '2' in
                # occurring_month. Hence doing that in Python
                # If month equals 'all',  accept all estimates for the category
                if month == 'all' or month in estimate.occurring_month:
                    estimate_found = True
                    actual_amount = estimate.actual_amount(start_date,
                                                           end_date,
                                                           transactions)
                    actual_total += actual_amount
                    categories_estimates_and_transactions.append({
                        'category': category,
                        'estimate': estimate,
                        'transactions': estimate.actual_transactions(start_date,
                                                                     end_date),
                        'actual_amount': actual_amount,
                    })
                if not estimate_found:
                    # Set estimate and transactions to empty query set if no
                    # estimate found
                    categories_estimates_and_transactions.append({
                        'category': category,
                        'estimate': self.estimates.none(),
                        'transactions': Transaction.objects.none(),
                        'actual_amount': actual_amount,
                    })

        return (categories_estimates_and_transactions, actual_total)

    def category_estimates(self, cat_query):
        """
        Returns a query list of estimates for certain category

        cat_query: Q() object containing wanted category 'category=<wanted_cat'
        """

        return (self.estimates.filter(cat_query).exclude(is_deleted=True))

    def actuals(self, start_date, end_date, estimate):
        """

        """
        estimates_and_actuals = []
        actual_amount = None
        actual_amount = estimate.actual_amount(start_date, end_date)
        estimates_and_actuals.append({
            'estimate': estimate,
            'actual_amount': actual_amount,
        })

    def estimates_and_actuals(self, start_date, end_date,
                              cat_query, occurrence_query_list=Q()):
        # Search for each category, and where occurence match
        query_list = (cat_query & occurrence_query_list)
        estimates_and_actuals = []
        actual_total = Decimal('0.0')
        estimate_found = False
        actual_amount = None

        for estimate in self.category_estimates(query_list):
            estimate_found = True
            actual_amount = estimate.actual_amount(start_date, end_date)
            actual_total += actual_amount
            estimates_and_actuals.append({
                'estimate': estimate,
                'actual_amount': actual_amount,
            })
        if not estimate_found:
            # Set estimate and transactions to empty query set if no
            # estimate found
            estimates_and_actuals.append({
                'estimate': self.estimates.none(),
                'actual_amount': actual_amount,
            })

        return (estimates_and_actuals, actual_total)

    def actual_total(self, start_date, end_date):
        actual_total = Decimal('0.0')

        for estimate in self.estimates.exclude(is_deleted=True):
            actual_amount = estimate.actual_amount(start_date, end_date)
            actual_total += actual_amount

        return actual_total

    def yearly_data_per_category(self, categories, budget, year):
        """
        :param budget.categories.Category categories: Category list
        :param budget.Budget budget: Budget to work on
        :param str year: Year of the budget
        :return: yearly estimates_and_actuals, actual_yearly_total, start_date
        """

        estimates_and_actuals = []

        # Total amount of money used in a year
        actual_yearly_total = Decimal(0.0)

        YearlyData = namedtuple('YearlyData',
                                ['estimates_and_actuals',
                                 'actual_yearly_total'])

        for category in categories:
            # Total used in year per category
            actual_yearly_total_in_cat = {}
            actual_monthly_total = {}
            actual_monthly_total[category] = Decimal(0.0)
            estimated_yearly_total = {}
            estimated_yearly_total[category] = Decimal(0.0)
            actual_monthly_total[category] = Decimal(0.0)
            actual_yearly_total_in_cat[category] = Decimal(0.0)

            monthly_data_per_category = []
            monthly_data_per_category.append(category)
            monthly_data = []

            category_query = Q(category=category)
            estimates = budget.category_estimates(category_query)
            transactions =  Transaction.expenses.filter(category=category).order_by('date')

            for month_number, month_name in enumerate(calendar.month_name):
                #
                actual_monthly_total[category] = Decimal(0.0)

                # month number 0 is empty string
                if month_number == 0:
                    continue

                start_date = datetime.date(int(year), month_number, 1)
                end_date = datetime.date(int(year),
                                         month_number,
                                         calendar.monthrange(int(year),
                                                             month_number)[1])
                for estimate in estimates:

                    estimated_yearly_amount = estimate.yearly_estimated_amount()

                    # Need refactoring, a better way to get monthly
                    # estimates for a category than just divide by 12
                    estimated_yearly_total[category] += estimated_yearly_amount / Decimal(12)

                    # estimate.actual_amount return all transaction for the
                    # category
                    # Even if there's multiple estimates for the same
                    # category, actual_amount needs to be calculated only once
                    if actual_monthly_total[category] == Decimal(0.0):
                        actual_monthly_total[category] = estimate.actual_amount(
                            start_date,
                            end_date,
                            transactions)

                    # Get estimates and actuals for the date range
                    # eaa, actual_monthly_total_cat =
                    # budget.estimates_and_actuals(start_date, end_date,
                    # category_query, Q())
                actual_yearly_total_in_cat[category] += actual_monthly_total[category]
                monthly_data.append({
                    'actual_monthly_total_in_category': actual_monthly_total[category],
                })

            actual_yearly_total += actual_yearly_total_in_cat[category]

            monthly_data_per_category.append(monthly_data)

            # Total per category within year
            monthly_data_per_category.append(actual_yearly_total_in_cat[category])
            monthly_data_per_category.append(estimated_yearly_total[category])
            # Store monthly data for current category
            estimates_and_actuals.append(monthly_data_per_category)

        return YearlyData(estimates_and_actuals=estimates_and_actuals,
                          actual_yearly_total=actual_yearly_total)

    class Meta:
        verbose_name = _('Budget')
        verbose_name_plural = _('Budgets')


class BudgetEstimate(StandardMetadata):
    """
    The individual line items that make up a budget.

    Some examples include possible items like "Mortgage", "Rent", "Food",
    "Misc" and "Car Payment".
    """

    REPEAT_CHOICES = (
        ('BIWEEKLY', _('Every 2 Weeks')),
        ('MONTHLY', _('Every Month')),
    )

    JANUARY = 1
    FEBRUARY = 2
    MARCH = 3
    APRIL = 4
    MAY = 5
    JUNE = 6
    JULY = 7
    AUGUST = 8
    SEPTEMBER = 9
    OCTOBER = 10
    NOVEMBER = 11
    DECEMBER = 12

    MONTH_CHOICES = (
        (JANUARY, _('January')),
        (FEBRUARY, _('February')),
        (MARCH, _('March')),
        (APRIL, _('April')),
        (MAY, _('May')),
        (JUNE, _('June')),
        (JULY, _('July')),
        (AUGUST, _('August')),
        (SEPTEMBER, _('September')),
        (OCTOBER, _('October')),
        (NOVEMBER, _('November')),
        (DECEMBER, _('December')),
    )

    budget = models.ForeignKey(Budget,
                               related_name='estimates',
                               verbose_name=_('Budget'),
                               on_delete=models.CASCADE)
    category = models.ForeignKey(Category,
                                 related_name='estimates',
                                 verbose_name=_('Category'),
                                 on_delete=models.CASCADE)
    amount = models.DecimalField(_('Amount'), max_digits=11, decimal_places=2)
    occurring_month = MultiSelectField(_('Occuring month'),
                                       null=False, blank=False,
                                       choices=MONTH_CHOICES,
                                       default=u'1')

    objects = models.Manager()
    active = ActiveManager()

    def __unicode__(self):
        return u"%s - %s" % (self.category.name, self.amount)

    def yearly_estimated_amount(self):

        yearly_estimate = Decimal(0.0)
        for month in self.occurring_month:
            yearly_estimate += self.amount

        return yearly_estimate

    def actual_transactions(self, start_date=None, end_date=None):
        # Estimates should only report on expenses to prevent incomes from
        # (incorrectly) artificially inflating totals.
        if (start_date and end_date):
            return Transaction.expenses.filter(category=self.category, date__range=(start_date, end_date)).order_by('date').select_related('category')
        return Transaction.expenses.filter(category=self.category).order_by('date').select_related('category')

    def actual_amount(self, start_date, end_date, transactions=None):
        """Return actual spendings during the period

        :param start_date: start of range
        :param end_date: end of range
        :param transaction: queryset of all expenses in certain category

        Doing the date filtering in Python to avoid hitting database
        hundreds of times (this function is 12 (number of months in year) *
        number of categories).
        """

        total = Decimal('0.0')
        if transactions:
            for transaction in transactions:
                if start_date <= transaction.date <= end_date:
                    total += transaction.amount
        return total


    class Meta:
        verbose_name = _('Budget estimate')
        verbose_name_plural = _('Budget estimates')



import datetime
from decimal import Decimal

from django.db import models
from django.utils.translation import ugettext_lazy as _

from budget.categories.models import Category
from budget.base_models import ActiveManager, StandardMetadata


TRANSACTION_TYPES = (
    ('expense', _('Expense')),
    ('income', _('Income')),
)


class TransactionManager(ActiveManager):
    def get_latest(self, limit=10):
        return self.get_queryset().order_by('-date', '-created')[0:limit]

    def get_all(self, start_date, end_date):
        """
        Return queryset of all transactions between start_data and end_date
        """

        # Estimates should only report on expenses to prevent incomes from
        # (incorrectly) artificially inflating totals.
        return self.get_queryset().filter(date__range=(start_date, end_date)).order_by('date').select_related('budget')


class TransactionExpenseManager(TransactionManager):
    def get_queryset(self):
        return super(TransactionExpenseManager, self).get_queryset().filter(transaction_type='expense').select_related('category')

    def actual_transactions(self, start_date, end_date):
        """
        Return queryset of all expenses between start_date and end_date
        """

        return Transaction.expenses.filter(date__range=(start_date, end_date)).order_by('date')

    def actual_amount(self, start_date, end_date):
        """
        Sum of all expenses between given dates
        """

        total = Decimal('0.0')
        for transaction in self.actual_transactions(start_date, end_date):
            total += transaction.amount
        return total

    def actual_transactions_in_category(self, category, start_date, end_date):
        """
        Return queryset of all transactions for given gategory between
        given dates
        """

        return Transaction.expenses.filter(category=category, date__range=(start_date, end_date)).order_by('date')

    def actual_amount_in_category(self, category, start_date, end_date):
        """
        Sum of all transactions in given category between given dates
        """
        total = Decimal('0.0')
        for transaction in self.actual_transactions_in_category(category, start_date, end_date):
            total += transaction.amount
        return total


class TransactionIncomeManager(TransactionManager):
    def get_queryset(self):
        return super(TransactionIncomeManager, self).get_queryset().filter(transaction_type='income')


class Transaction(StandardMetadata):
    """
    Represents incomes/expenses for the party doing the budgeting.

    Transactions are not tied to individual budgets because this allows
    different budgets to applied (like a filter) to a set of transactions.
    It also allows for budgets to change through time without altering the
    actual incoming/outgoing funds.
    """
    transaction_type = models.CharField(_('Transaction type'),
                                        max_length=32,
                                        choices=TRANSACTION_TYPES,
                                        default='expense', db_index=True)
    notes = models.CharField(_('Notes'), max_length=255, blank=True)
    category = models.ForeignKey(Category,
                                 related_name='transactions',
                                 verbose_name=_('Category'),
                                 on_delete=models.CASCADE)
    amount = models.DecimalField(_('Amount'), max_digits=11, decimal_places=2)
    date = models.DateField(_('Date'), default=datetime.date.today,
                            db_index=True)

    objects = models.Manager()
    active = ActiveManager()
    expenses = TransactionExpenseManager()
    incomes = TransactionIncomeManager()

    def __unicode__(self):
        return u"%s (%s) - %s" % (self.notes,
                                  self.get_transaction_type_display(),
                                  self.amount)

    class Meta:
        verbose_name = _('Transaction')
        verbose_name_plural = _('Transactions')



import datetime

#
from django import forms
from django.template.defaultfilters import slugify

#
from mptt.forms import TreeNodeChoiceField

#
from budget.models import Budget, BudgetEstimate
from budget.categories.models import Category

class BudgetForm(forms.ModelForm):
    start_date = forms.SplitDateTimeField(initial=datetime.datetime.now, required=False, widget=forms.SplitDateTimeWidget)
    
    class Meta:
        model = Budget
        fields = ('name', 'start_date')
    
    def save(self):
        if not self.instance.slug:
            self.instance.slug = slugify(self.cleaned_data['name'])
        super(BudgetForm, self).save()


class BudgetEstimateForm(forms.ModelForm):

    error_css_class = 'alert alert-danger'
    required_css_class = 'alert alert-info'
    # Replace category in the model with hieararchial category list
    category = TreeNodeChoiceField(queryset=Category.objects.all())

    class Meta:
        model = BudgetEstimate
        fields = ('category', 'amount', 'occurring_month')
    
    def save(self, budget):
        self.instance.budget = budget
        super(BudgetEstimateForm, self).save()

    def clean(self):
        """
        Check if either 'repeat' or 'occurring_month' is given, but not
        both 
        """

        cleaned_data = super(BudgetEstimateForm, self).clean()
        repeat = cleaned_data.get('repeat')
        occurs = cleaned_data.get('occurring_month')

        if repeat and occurs:
            msg = u'Must select only one: Repeat or Occurring month'
            self._errors['repeat'] = self.error_class([msg])
            self._errors['occurring_month'] = self.error_class([msg])
            del cleaned_data['repeat']
            del cleaned_data['occurring_month']

        elif not repeat and not occurs:
            msg = u'Must select one: Repeat or Occurring month'
            self._errors['repeat'] = self.error_class([msg])
            self._errors['occurring_month'] = self.error_class([msg])
            del cleaned_data['repeat']
            del cleaned_data['occurring_month']
            
        return cleaned_data



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import multiselectfield.db.fields


class Migration(migrations.Migration):

    dependencies = [
        ('budget', '0003_auto_20151218_0740'),
    ]

    operations = [
        migrations.AlterField(
            model_name='budgetestimate',
            name='occurring_month',
            field=multiselectfield.db.fields.MultiSelectField(default='1', max_length=26, verbose_name='Occuring month', choices=[(1, 'January'), (2, 'February'), (3, 'March'), (4, 'April'), (5, 'May'), (6, 'June'), (7, 'July'), (8, 'August'), (9, 'September'), (10, 'October'), (11, 'November'), (12, 'December')]),
        ),
    ]



from django.contrib import admin
#
from mptt.admin import MPTTModelAdmin
#
from budget.categories.models import Category

class CategoryAdmin(admin.ModelAdmin):
    fieldsets = (
        (None, {
            'fields': ('name', 'slug'),
        }),
        ('Metadata', {
            'classes': ('collapse',),
            'fields': ('created', 'updated', 'is_deleted')
        })
    )
    list_display = ('name', 'is_deleted')
    list_filter = ('is_deleted',)
    prepopulated_fields = {
        'slug': ('name',),
    }
    search_fields = ('name',)


admin.site.register(Category, MPTTModelAdmin)



import datetime

from django.db import models
from django.utils.translation import ugettext_lazy as _

class ActiveManager(models.Manager):
    def get_queryset(self):
        return super(ActiveManager, self).get_queryset().filter(is_deleted=False)

class StandardMetadata(models.Model):
    """
    A basic (abstract) model for metadata.
    """
    created = models.DateTimeField(_('Created'), default=datetime.datetime.now)
    updated = models.DateTimeField(_('Updated'), default=datetime.datetime.now)
    is_deleted = models.BooleanField(_('Is deleted'), default=False, db_index=True)
    
    class Meta:
        abstract = True
    
    def save(self, *args, **kwargs):
        self.updated = datetime.datetime.now()
        super(StandardMetadata, self).save(*args, **kwargs)
    
    def delete(self):
        self.is_deleted = True
        self.save()




from decimal import Decimal
from django import template
from django.conf import settings


register = template.Library()

# To override, copy to your settings file. Make sure to keep the tuples in 
# descending order by percentage.
BUDGET_DEFAULT_COLORS = (
    # (percentage, CSS color class)
    (1.001, 'red'),
    (0.75, 'yellow'),
    (0.0, 'green'),
)


class ColorizeAmountNode(template.Node):
    def __init__(self, estimated_amount, actual_amount):
        self.estimated_amount = template.Variable(estimated_amount)
        self.actual_amount = template.Variable(actual_amount)
    
    def render(self, context):
        if hasattr(settings, 'BUDGET_DEFAULT_COLORS'):
            colors = settings.BUDGET_DEFAULT_COLORS
        else:
            colors = BUDGET_DEFAULT_COLORS
        
        try:
            estimate = self.estimated_amount.resolve(context)
            actual = self.actual_amount.resolve(context)
            estimate = make_decimal(estimate)

            if estimate == 0:
                return ''

            actual = make_decimal(actual)            
            percentage = actual / estimate
            
            for color in colors:
                color_percentage = make_decimal(color[0])
                
                if percentage >= color_percentage:
                    return color[1]
        except template.VariableDoesNotExist:
            return ''


def make_decimal(amount):
    """
    If it's not a Decimal, it should be...
    """
    if not isinstance(amount, Decimal):
        amount = Decimal(str(amount))
    
    return amount
    

def colorize_amount(parser, token):
    """
    Compares an estimate with an actual amount and returns an appropriate
    color as a visual indicator.
    
    Example:
    
        {% colorize_amount estimated_amount actual_amount %}
    """
    try:
        tag_name, estimated_amount, actual_amount = token.split_contents()
    except ValueError:
        raise template.TemplateSyntaxError("%r tag requires exactly two arguments" % token.contents.split()[0])
    return ColorizeAmountNode(estimated_amount, actual_amount)


register.tag('colorize_amount', colorize_amount)



from django.contrib import admin
from budget.transactions.models import Transaction


class TransactionAdmin(admin.ModelAdmin):
    date_hierarchy = 'date'
    fieldsets = (
        (None, {
            'fields': ('transaction_type', 'notes', 'category', 'amount', 'date'),
        }),
        ('Metadata', {
            'classes': ('collapse',),
            'fields': ('created', 'updated', 'is_deleted')
        })
    )
    list_display = ('notes', 'transaction_type', 'amount', 'date', 'is_deleted')
    list_filter = ('is_deleted',)
    search_fields = ('notes',)


admin.site.register(Transaction, TransactionAdmin)
